# Generated by Django 5.1.7 on 2025-04-10 10:10

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ResilientLogEntry",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("is_sent", models.BooleanField(default=False, verbose_name="is sent")),
                ("level", models.IntegerField(default=0, verbose_name="level")),
                ("message", models.JSONField(verbose_name="message")),
                ("context", models.JSONField(null=True, verbose_name="context")),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="created at"),
                ),
            ],
            options={
                "verbose_name_plural": "resilient log entries",
            },
        ),
    ]
