# Generated by Django 4.2.25 on 2025-10-29 07:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resilient_logger", "0003_created_at_sorting_and_index"),
    ]

    operations = [
        migrations.AlterField(
            model_name="resilientlogentry",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
