# Generated by Django 4.2.25 on 2025-10-29 06:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resilient_logger", "0002_alter_verbose_name"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="resilientlogentry",
            options={
                "ordering": ["-created_at", "-id"],
                "verbose_name": "resilient log entry",
                "verbose_name_plural": "resilient log entries",
            },
        ),
        migrations.AlterField(
            model_name="resilientlogentry",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True, db_index=True, verbose_name="created at"
            ),
        ),
    ]
