# Generated by Django 4.2.25 on 2025-10-29 05:35

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resilient_logger", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="resilientlogentry",
            options={
                "verbose_name": "resilient log entry",
                "verbose_name_plural": "resilient log entries",
            },
        ),
    ]
